{
  	viewer {
    		savingsAccount {
      			redeemableDeposits {
      				id
              rate
              vehicle
              openDate
              maturityDate
              principal
              redeemedBalance{
                  netAmount
                  yield
                  incomeTax
                  iofTax
                  id
              }
    			}
  		}
  	}
}
